local resourceRoot = getResourceRootElement ( getThisResource() )
local root = getRootElement()
local scrollFields = { "height", "width", "thickness", "fog", "count" }
local elementData = {}
local guiElements = {}

function setLocalElementData ( element, field, value )
	if ( not elementData[element] ) then
		elementData[element] = {}
	end
	elementData[element][field] = value
end

function getLocalElementData ( element, field )
	return elementData[element][field]
end

function getRange ( field )
	if field == "height" then
		return 1, 30
	elseif field == "width" then
		return 1, 20
	elseif field == "thickness" then
		return 0.1, 2
	elseif field == "fog" then
		return 0, 1
	elseif field == "count" then
		return 1, 1000
	end
end 

function getOption ( field ) 
	if field == "height" then
		return 20
	elseif field == "width" then
		return 10
	elseif field == "thickness" then
		return 0.1
	elseif field == "fog" then
		return 0.05
	elseif field == "count" then
		return 100
	end
end 

--Whole naming system is interlinked so I'm just injecting hax here with the below table and function
labelName = { "Height", "Cone Shape", "Diameter", "Fog Object %", "Total Objects" } 

function crossReference(field)
	for k,v in pairs(scrollFields) do
		if field == scrollFields[k] then
			return k
		end
	end	
end
-----------------------------------------------------------------------------------------------------

function createScroll ( x, y, width, height, maxLength, space, window, field )
	local begin, finish = getRange ( field )
	local default = getOption ( field )
	local scrollBar = guiCreateScrollBar ( x, y, width, height, true, true, window )
	
	guiSetProperty ( scrollBar, "StepSize", ".01" )
	local scrollBox = guiCreateEdit ( 0.8, y, 0.16, height, tostring ( default ) , true, window )
	local percentage = getNewPercentage ( field, default )
	guiScrollBarSetScrollPosition ( scrollBar, percentage )
	guiEditSetMaxLength ( scrollBox, maxLength )
	addEventHandler ( "onClientGUIScroll", scrollBar, checkBar, false )
	addEventHandler ( "onClientGUIChanged", scrollBox, checkBox, false )	
	local name = guiCreateLabel ( x + 0.05, y - ( space / 2 ), 0.3, height, labelName[crossReference(field)], true, window )
	local label_01 = guiCreateLabel ( x - 0.1, y, 0.09, height, tostring ( begin ), true, window )
	local label_02 = guiCreateLabel ( x + width, y, 0.09, height, tostring ( finish ), true, window )
	guiLabelSetVerticalAlign ( name, "center" )
	guiLabelSetVerticalAlign ( label_01, "center" )
	guiLabelSetVerticalAlign ( label_02, "center" )
	guiLabelSetHorizontalAlign ( name, "left" )
	guiLabelSetHorizontalAlign ( label_01, "right" )
	guiLabelSetHorizontalAlign ( label_02, "right" )
	setLocalElementData ( scrollBar, "field", field )
	setLocalElementData ( scrollBar, "box", scrollBox )
	setLocalElementData ( scrollBar, "name", label_02 )
	setLocalElementData ( scrollBar, "label_01", label_01 )
	setLocalElementData ( scrollBar, "label_02", label_02 )
	return scrollBar
end

function createWindow()
	local window = guiCreateWindow ( 0.62, 0.25, 0.35, 0.55, "Tornado Options", true )
	showCursor ( true )
	toggleAllControls ( false, false, true )
	guiWindowSetMovable ( window, true )
	guiWindowSetSizable ( window, false )
	bindKey ( "m", "down", toggleWindow )
	guiElements.window = window
	guiElements.isWindowActive = true
	
	-- local editBox = guiCreateEdit ( 0.3, 0.085, 0.4, 0.08, "", true, window )
	-- guiCreateLabel ( 0.07, 0.1, 0.3, 0.2, "Name: ", true, window )
	
	guiEditSetMaxLength ( editBox, 24 )
	guiElements.editBox = editBox
	
	local closeButton = guiCreateButton ( 0.89, 0.085, 0.08, 0.08, "x", true, window )
	addEventHandler ( "onClientGUIClick", closeButton, hideWindow, false )
	guiElements.closeButton = closeButton
	
	local scrollTable = {}
	local scrolls = 5
	local space = 0.8 / ( scrolls + 1 )
	for i=1, scrolls do
		local y = 0.1 + i * space
		local scroll = createScroll ( 0.1, y, 0.55, 0.06, 6, space, window, scrollFields[i] )
		table.insert ( scrollTable, scroll )
	end
	guiElements.scrollTable = scrollTable
end

function getNewPosition ( field, percentage )
	local begin, finish = getRange ( field )
	local newPosition = begin + ( finish - begin ) * percentage / 100
	return ( field == "count" ) and math.floor ( newPosition ) or newPosition
end

function getNewPercentage ( field, position )
	local begin, finish = getRange ( field )
	local newPercentage = ( position - begin ) * 100 / ( finish - begin )
	return newPercentage
end

function getScrollBar ( scrollBox )
	local scrollBar = nil
	for i, v in ipairs ( guiElements.scrollTable ) do
		local box = getLocalElementData ( v, "box" )
		if ( scrollBox == box ) then
			scrollBar = v
			break
		end
	end
	return scrollBar
end

-- string.format( "%.4f", 0.023232323 ) -- 4 places
function roundToString ( num, places )
	num = tostring ( num )
	if num:match ( '%.' ) then
		num = num:gsub ( '(%.' .. ( '%d' ):rep ( places ) .. ')%d+', '%1' )
	end
	return num
end

function checkBar ( scrollBar )
	local field = getLocalElementData ( scrollBar, "field" )
	if field == "fog" or field == "count" then
		if ( not guiElements.isInUse ) then
			local scrollBox = getLocalElementData ( scrollBar, "box" )
			local field = getLocalElementData ( scrollBar, "field" )
			local percentage = guiScrollBarGetScrollPosition ( scrollBar )
			local position = getNewPosition ( field, percentage )
			
			guiElements.isInUse = true
			guiSetText ( scrollBox, roundToString ( position, 2 ) )
			setOption ( field, position )
			guiElements.isInUse = false
		end	
		
		if not regenerateDelay then
			destroyTornado()
			if pieces then
				if pieces.count then
					for i=1, pieces.count do
						if pieces.createPieceDelay then
							killTimer ( pieces.createPieceDelay )
							outputChatBox ( "got it." )
						end
					end
				end
			end
			regenerateDelay = setTimer ( function ()
				onStart()		
				regenerateDelay = nil
				-- outputChatBox ( "regenerateDelay ENDED" ) --DEBUG
			end, 500, 1 )
		end
	else
		if ( not guiElements.isInUse ) then
			local scrollBox = getLocalElementData ( scrollBar, "box" )
			local field = getLocalElementData ( scrollBar, "field" )
			local percentage = guiScrollBarGetScrollPosition ( scrollBar )
			local position = getNewPosition ( field, percentage )
			
			guiElements.isInUse = true
			guiSetText ( scrollBox, roundToString ( position, 2 ) )
			setOption ( field, position )
			guiElements.isInUse = false
		end
	end
end

function checkBox ( scrollBox )
	
	if ( not guiElements.isInUse ) then
		local scrollBar = getScrollBar ( scrollBox )
		local field = getLocalElementData ( scrollBar, "field" )
		local percentage = guiScrollBarGetScrollPosition ( scrollBar )
		local position = tonumber ( guiGetText ( scrollBox ) )
		
		guiElements.isInUse = true
		if ( position ) then
			local begin, finish = getRange ( field )
			if ( position >= begin ) and ( position <= finish ) then
				percentage = getNewPercentage ( field, position )
				guiScrollBarSetScrollPosition ( scrollBar, percentage )
				setOption ( field, position )
			else
				position = getNewPosition ( field, percentage )
				guiSetText ( scrollBox, roundToString ( position, 2 ) )
			end
		else
			position = getNewPosition ( field, percentage )
			guiSetText ( scrollBox, roundToString ( position, 2 ) )
		end
		guiElements.isInUse = false
	end
end

function toggleWindow()
	local active = guiElements.isWindowActive
	if ( active ) then
		hideWindow ( "left" )
	else
		showWindow()
	end
end    

function hideWindow ( button )
	if ( button == "left" ) then
		local window = guiElements.window
		showCursor ( false )
		guiSetVisible ( window, false )
		toggleAllControls ( true )
		guiElements.isWindowActive = false
	end
end

function showWindow()
	local window = guiElements.window
	showCursor ( true )
	guiSetVisible ( window, true )
	toggleAllControls ( false, false, true )
	guiElements.isWindowActive = true
end

addEvent ( "onClientWindowReady" )
addEventHandler ( "onClientWindowReady", root, createWindow )


createWindow()